#!/usr/bin/env python
"""

"""
# Author: Harco Kuppens


import sys, re, pprint  # modules from standard lib (python 2.6 and later)
import yaml
import argparse
import tempfile
import shutil


import uppaal2bcg

# load libraries from ./lib
import lib.util,lib.check,lib.run,lib.error

#import lts2lts


# explicit define public api for when used as module
__all__ = [ 'compare' ]
# note: main and argparsing are just helpers for the function give_user_acces_to_dir_using_acl


def args_parsing(args):
    import os.path
    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('--keep-tmp-files', action='store_true', help='keep temporary files generated for doing comparison')
    argparser.add_argument('modelfile1', help='model file which can be one of : uppaal model,cadp aut, jtorx aut, dot')
    argparser.add_argument('modelfile2', help='model file which can be one of : uppaal model,cadp aut, jtorx aut, dot')
    argparser.add_argument('range_min', help='minimum value of range used for input params when flattening the model')
    argparser.add_argument('range_max', help='maximum value of range used for input params when flattening the model')

    parse_result=argparser.parse_args(args[1:])

    keeptmp=parse_result.keep_tmp_files

    # handle input/output file argument
    modelfile1=os.path.abspath(parse_result.modelfile1).replace('\\','/')
    modelfile2=os.path.abspath(parse_result.modelfile2).replace('\\','/')
    # Note: on windows also interprets / as dir sep, and returns path with windows \


    lib.check.check_isreadablefile(modelfile1)
    lib.check.check_isreadablefile(modelfile2)

    range_min=parse_result.range_min
    range_max=parse_result.range_max

    return  [modelfile1,modelfile2,range_min,range_max,keeptmp]


def model2bcg(input,outdir,name,range_min,range_max,keeptmp):

    output_str=""

    bcgfile=outdir + '/' + name + '.bcg'
    if input.endswith('xml'):
       output_str+=uppaal2bcg.uppaal2bcg(input,bcgfile,range_min,range_max,keeptmp)

    if input.endswith('bcg'):
       shutil.copy(input,bcgfile)

# TODO: add other lts types with nice lts2lts script
#    if input.endswith('jtorx.aut'):
#       cadp_aut_file=outdir + '/' + name + '.cadp.aut'
#       jtorx_aut2cadp_aut.jtorx_aut2cadp_aut(input,cadp_aut_file)
#       lib.run.run_command([ 'bcg_io',  cadp_aut_file,bcgfile ],cwd=outdir)
#    if input.endswith('cadp.aut'):
#       lib.run.run_command([ 'bcg_io',  input,bcgfile ],cwd=outdir)
#    if input.endswith('dot'):
#       jtorx_aut_file=outdir + '/' + name + '.jtorx.aut'
#       learnlib_dot2jtorx_aut.dot2jtorxaut(input,jtorx_aut_file)
#       output_str+=model2bcg(jtorx_aut_file,outdir,name,range_min,range_max,keeptmp)

    return output_str

def compare(modelfile1,modelfile2,range_min,range_max,keeptmp=False):
    tmpdir=tempfile.mkdtemp(prefix="tomte_compare_")
    line="\n-----------------------------------------------\n" 
    output_str="flatten first model to a cadp bcg file" 
    output_str+=line
    output_str+=model2bcg(modelfile1,tmpdir,'model1',range_min,range_max,keeptmp)
    output_str+="flatten second model to a cadp bcg file" 
    output_str+=line
    output_str+=model2bcg(modelfile2,tmpdir,'model2',range_min,range_max,keeptmp)

    output_str+="compare both models with cadp caesar" 
    output_str+=line
    output_str+=lib.run.run_command_with_output(['time', 'bcg_open', 'model1.bcg', 'bisimulator', '-diag', '-bfs', '-strong',  'model2.bcg'],cwd=tmpdir)

    #print "results in directory: " +  tmpdir
    if not keeptmp: 
       shutil.rmtree(tmpdir)

    return output_str

def main(args):
    """

    """
    (modelfile1,modelfile2,range_min,range_max,keeptmp)=args_parsing(args)


    print compare(modelfile1,modelfile2,range_min,range_max,keeptmp)



if __name__ == "__main__":
    try:
        main(sys.argv)
    except KeyboardInterrupt as e:
        print >> sys.stderr, "\n\nExiting on user cancel."
        sys.exit(1)

