#!/usr/bin/env python
"""
   generate Sut.java class implementation from uppaal model

   Generate Sut.java class implementation from uppaal model.
   Note this only generates java class doing basicly the Sut implemention,
   but doesn't delivers you all java classes it depends on. To get the 
   whole Sut implementation bundle, that is with dependencies, use the 
   sut_uppaal2java command instead.
"""
# Author: Harco Kuppens


import sys, os,  pprint, re, getpass, tempfile   # standard libraries
import argparse # only in standard lib for python 2.7 and later -> installed local copy  (pure python versions)


import model_parser
import gentemplate
import util









def args_parsing(args):
    """
    parses args for following usage :

         <script_name> model
    
    if it doesn't match this usage description, then it displays
    this description with a listing of all possible models
    in the 'models_dir' 
    
    """

    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('modelfile', help='see below for available models')
    argparser.add_argument('sutjavafile', help='the to be generated sut java file ')

    str=argparser.format_help()

    def myhelp( ):
        print str  + "\n"

    def myerror( message):
            print >>sys.stderr, ('%s: error: %s\n') % (argparser.prog, message)
            argparser.print_usage(sys.stderr)
            exit(1)

    argparser.error=myerror
    argparser.format_help=myhelp

    parse_result=argparser.parse_args(args[1:])

    # handle modelfile argument
    model_file=parse_result.modelfile
    # if model_file is not a file path to a xml file, we assume
    # only the model's name is give as input!
    if not model_file.endswith('.xml'):
       model_file= models_dir  + '/' + model_file + ".xml"
    model_file=os.path.abspath(model_file).replace('\\','/')
    # always use unix slashes in python!!
    # automatic convert any windows slashes
    # TODO: document this convention in python/txtdocs/

    # handle modelfile argument
    sut_java_file=os.path.abspath(parse_result.sutjavafile).replace('\\','/')

    util.check_iswritabledir(os.path.dirname(sut_java_file))

    return  [model_file,sut_java_file]


def uppaal2sutjavafile(model_file,sut_java_file):

    # template dir is relative from this module :
    template_dir = os.path.dirname(os.path.abspath(__file__))   + '/templates'

    # generate files in tmpdir
    tmpdir=tempfile.mkdtemp(prefix="sut_uppaal2sut_")
    # -output config file
    config_file=  os.path.abspath(tmpdir + "/config.yaml")

    # date from model
    parsed_data=model_parser.parse_uppaal_model(model_file)
    parsed_data['model_file']=model_file.replace('\\','\\\\')

    util.write_config(config_file,parsed_data)
    #print "config written: " + config_file

    gentemplate.fill_template(sut_java_file,parsed_data,template_dir,'Sut.java')
    #print "sut written: " + sut_java_file



def main(args):
    """
     usage: sut_uppaal2sutjavafile model javafile   

     note:  java file is generate at the given file location which
            may not match with the package 'sut.implementation'
    """  
    (model_file,sut_java_file)=args_parsing(args)
    uppaal2sutjavafile(model_file,sut_java_file)


if __name__ == "__main__":
    main(sys.argv)



